@@ -5,6 +5,8 @@ use std::io::{self, BufReader, Read};
 use std::path::PathBuf;
 use structopt::StructOpt;
 mod datatype;
+use calm_io::stdout;
+use calm_io::stdoutln;
 use crossterm::terminal::size;
 use directories::BaseDirs;
 use serde::Deserialize;
@@ -49,6 +51,8 @@ use toml;
         #std_color = [133, 205, 202]
         ## na_color = [R,G,B] color for NA values
         #na_color = [226, 125, 95]
+        ## neg_num_color = [R,G,B] color for negative values
+        #neg_num_color = [226, 125, 95]
 "
 )]
 struct Cli {
@@ -102,18 +106,17 @@ struct Cli {
     #[structopt(
         short = "s",
         long = "delimiter",
-        default_value = ",",
         parse(try_from_str = datatype::parse_delimiter),
         help = "The delimiter separating the columns."
     )]
-    delimiter: u8,
-    //#[structopt(
-    //    short = "sig",
-    //    long = "sigfig",
-    //    default_value = "3",
-    //    help = "Significant Digits. Default 3. (Comming Soon!)"
-    //)]
-    //sigfig: usize,
+    delimiter: Option<u8>,
+    #[structopt(
+        short = "g",
+        long = "sigfig",
+        default_value = "3",
+        help = "Significant Digits. Default 3. Max is 7"
+    )]
+    sigfig: i64,
     #[structopt(
         short = "d",
         long = "debug-mode",
@@ -139,6 +142,7 @@ fn main() {
         header_color: toml::value::Array,
         std_color: toml::value::Array,
         na_color: toml::value::Array,
+        neg_num_color: toml::value::Array,
     }
 
     let base_dir = BaseDirs::new();
@@ -157,11 +161,16 @@ fn main() {
     let term_tuple = size().unwrap();
     let opt = Cli::from_args();
     let color_option = opt.color;
-    //let sigfig = opt.sigfig;
+    let sigfig = if opt.sigfig >= 3 && opt.sigfig <= 7 {
+        opt.sigfig
+    } else {
+        panic!("sigfig range must be between 3 and 7")
+    };
     let debug_mode = opt.debug_mode;
     let is_title_defined = opt.title.chars().count() > 0;
     let is_footer_defined = opt.title.chars().count() > 0;
     let is_row_display_defined = !(opt.row_display == 25);
+    let is_tty = atty::is(atty::Stream::Stdout);
 
     let title_option = match (&config, is_title_defined) {
         (Some(x), false) => &x.title,
@@ -188,21 +197,25 @@ fn main() {
     let nord_header_color: [u8; 3] = [94, 129, 172];
     let nord_std_color: [u8; 3] = [216, 222, 233];
     let nord_na_color: [u8; 3] = [191, 97, 106];
+    let nord_neg_num_color: [u8; 3] = [208, 135, 112];
     // one dark
     let one_dark_meta_color: [u8; 3] = [152, 195, 121];
     let one_dark_header_color: [u8; 3] = [97, 175, 239];
     let one_dark_std_color: [u8; 3] = [171, 178, 191];
     let one_dark_na_color: [u8; 3] = [224, 108, 117];
+    let one_dark_neg_num_color: [u8; 3] = [229, 192, 123];
     //// gruv
     let gruvbox_meta_color: [u8; 3] = [184, 187, 38];
     let gruvbox_header_color: [u8; 3] = [215, 153, 33];
     let gruvbox_std_color: [u8; 3] = [235, 219, 178];
     let gruvbox_na_color: [u8; 3] = [204, 36, 29];
+    let gruvbox_neg_num_color: [u8; 3] = [251, 73, 52];
     //// dracula
     let dracula_meta_color: [u8; 3] = [98, 114, 164];
     let dracula_header_color: [u8; 3] = [80, 250, 123];
     let dracula_std_color: [u8; 3] = [248, 248, 242];
     let dracula_na_color: [u8; 3] = [255, 121, 198];
+    let dracula_neg_num_color: [u8; 3] = [188, 63, 60];
 
     // user args
     let lower_column_width_defined = !(opt.lower_column_width == 2);
@@ -230,36 +243,41 @@ fn main() {
         upper_column_width
     };
     // logic for picking colors given config and user arguments
-    let (meta_color, header_color, std_color, na_color) = match color_option {
+    let (meta_color, header_color, std_color, na_color, neg_num_color) = match color_option {
         1 => (
             nord_meta_color,
             nord_header_color,
             nord_std_color,
             nord_na_color,
+            nord_neg_num_color,
         ),
         2 => (
             one_dark_meta_color,
             one_dark_header_color,
             one_dark_std_color,
             one_dark_na_color,
+            one_dark_neg_num_color,
         ),
         3 => (
             gruvbox_meta_color,
             gruvbox_header_color,
             gruvbox_std_color,
             gruvbox_na_color,
+            gruvbox_neg_num_color,
         ),
         4 => (
             dracula_meta_color,
             dracula_header_color,
             dracula_std_color,
             dracula_na_color,
+            dracula_neg_num_color,
         ),
         _ => (
             nord_meta_color,
             nord_header_color,
             nord_std_color,
             nord_na_color,
+            nord_neg_num_color,
         ),
     };
     let is_color_defined = opt.color > 0;
@@ -287,6 +305,12 @@ fn main() {
         (None, false) => nord_na_color,
         (None, true) => na_color,
     };
+    let neg_num_color = match (&config, is_color_defined) {
+        (Some(x), false) => get_color_from_config(&x.clone().neg_num_color),
+        (Some(_x), true) => neg_num_color,
+        (None, false) => nord_neg_num_color,
+        (None, true) => neg_num_color,
+    };
 
     //   colname reader
     let reader_result = build_reader(&opt);
@@ -353,7 +377,7 @@ fn main() {
     // vector of formatted values
     let vf: Vec<Vec<String>> = v
         .iter()
-        .map(|col| datatype::format_strings(col, lower_column_width, upper_column_width))
+        .map(|col| datatype::format_strings(col, lower_column_width, upper_column_width, sigfig))
         .collect();
 
     if debug_mode {
@@ -375,36 +399,102 @@ fn main() {
     // color
     let meta_text = "tv dim:";
     let div = "x";
-    print!("{: <6}", "");
-    println!(
-        "{} {} {} {}",
-        meta_text.truecolor(meta_color[0], meta_color[1], meta_color[2]),
-        (rows_in_file - 1).truecolor(meta_color[0], meta_color[1], meta_color[2]),
-        div.truecolor(meta_color[0], meta_color[1], meta_color[2]),
-        (cols).truecolor(meta_color[0], meta_color[1], meta_color[2]),
-    );
+    let _ = match stdout!("{: <6}", "") {
+        Ok(_) => Ok(()),
+        Err(e) => match e.kind() {
+            std::io::ErrorKind::BrokenPipe => Ok(()),
+            _ => Err(e),
+        },
+    };
+    if is_tty {
+        let _ = match stdoutln!(
+            "{} {} {} {}",
+            meta_text.truecolor(meta_color[0], meta_color[1], meta_color[2]),
+            (rows_in_file - 1).truecolor(meta_color[0], meta_color[1], meta_color[2]),
+            div.truecolor(meta_color[0], meta_color[1], meta_color[2]),
+            (cols).truecolor(meta_color[0], meta_color[1], meta_color[2]),
+        ) {
+            Ok(_) => Ok(()),
+            Err(e) => match e.kind() {
+                std::io::ErrorKind::BrokenPipe => Ok(()),
+                _ => Err(e),
+            },
+        };
+    } else {
+        let _ = match stdoutln!("{} {} {} {}", meta_text, rows_in_file - 1, div, cols) {
+            Ok(_) => Ok(()),
+            Err(e) => match e.kind() {
+                std::io::ErrorKind::BrokenPipe => Ok(()),
+                _ => Err(e),
+            },
+        };
+    }
     // title
     if !datatype::is_na(&title_option.clone()) {
-        print!("{: <6}", "");
-        println!(
-            "{}",
-            title_option
-                .truecolor(meta_color[0], meta_color[1], meta_color[2])
-                .underline()
-                .bold()
-        );
+        let _ = match stdout!("{: <6}", "") {
+            Ok(_) => Ok(()),
+            Err(e) => match e.kind() {
+                std::io::ErrorKind::BrokenPipe => Ok(()),
+                _ => Err(e),
+            },
+        };
+        if is_tty {
+            let _ = match stdoutln!(
+                "{}",
+                title_option
+                    .truecolor(meta_color[0], meta_color[1], meta_color[2])
+                    .underline()
+                    .bold()
+            ) {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
+        } else {
+            let _ = match stdoutln!("{}", title_option) {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
+        }
     }
 
     // header
-    print!("{: <6}", "");
+    let _ = match stdout!("{: <6}", "") {
+        Ok(_) => Ok(()),
+        Err(e) => match e.kind() {
+            std::io::ErrorKind::BrokenPipe => Ok(()),
+            _ => Err(e),
+        },
+    };
     //for col in 0..cols {
     for col in 0..num_cols_to_print {
         let text = vp[0].get(col).unwrap().to_string();
-        print!(
-            "{}",
-            text.truecolor(header_color[0], header_color[1], header_color[2])
-                .bold()
-        );
+        if is_tty {
+            let _ = match stdout!(
+                "{}",
+                text.truecolor(header_color[0], header_color[1], header_color[2])
+                    .bold()
+            ) {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
+        } else {
+            let _ = match stdout!("{}", text) {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
+        }
     }
     //println!();
     // datatypes
@@ -416,38 +506,108 @@ fn main() {
     //    owned_string.push_str(borrowed_string);
     //    print!("{}",owned_string.truecolor(143, 188, 187).bold());
     //}
-    println!();
+    let _ = match stdoutln!() {
+        Ok(_) => Ok(()),
+        Err(e) => match e.kind() {
+            std::io::ErrorKind::BrokenPipe => Ok(()),
+            _ => Err(e),
+        },
+    };
     vp.iter()
         .enumerate()
         .take(rows)
         .skip(1)
         .for_each(|(i, row)| {
-            print!(
-                "{: <6}",
-                i.truecolor(meta_color[0], meta_color[1], meta_color[2])
-            );
+            if is_tty {
+                let _ = match stdout!(
+                    "{: <6}",
+                    i.truecolor(meta_color[0], meta_color[1], meta_color[2])
+                ) {
+                    Ok(_) => Ok(()),
+                    Err(e) => match e.kind() {
+                        std::io::ErrorKind::BrokenPipe => Ok(()),
+                        _ => Err(e),
+                    },
+                };
+            } else {
+                let _ = match stdout!("{: <6}", i) {
+                    Ok(_) => Ok(()),
+                    Err(e) => match e.kind() {
+                        std::io::ErrorKind::BrokenPipe => Ok(()),
+                        _ => Err(e),
+                    },
+                };
+            }
             //for col in 0..cols {
             row.iter().take(num_cols_to_print).for_each(|col| {
-                print!(
-                    "{}",
-                    if datatype::is_na_string_padded(col) {
-                        col.truecolor(na_color[0], na_color[1], na_color[2])
-                    } else {
-                        col.truecolor(std_color[0], std_color[1], std_color[2])
-                    }
-                );
+                if is_tty {
+                    let _ = match stdout!(
+                        "{}",
+                        if datatype::is_na_string_padded(col) {
+                            col.truecolor(na_color[0], na_color[1], na_color[2])
+                        } else {
+                            if datatype::is_number(col) && datatype::is_negative_number(col) {
+                                col.truecolor(neg_num_color[0], neg_num_color[1], neg_num_color[2])
+                            } else {
+                                col.truecolor(std_color[0], std_color[1], std_color[2])
+                            }
+                        }
+                    ) {
+                        Ok(_) => Ok(()),
+                        Err(e) => match e.kind() {
+                            std::io::ErrorKind::BrokenPipe => Ok(()),
+                            _ => Err(e),
+                        },
+                    };
+                } else {
+                    let _ = match stdout!("{}", col) {
+                        Ok(_) => Ok(()),
+                        Err(e) => match e.kind() {
+                            std::io::ErrorKind::BrokenPipe => Ok(()),
+                            _ => Err(e),
+                        },
+                    };
+                }
             });
-            println!();
+            let _ = match stdoutln!() {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
         });
 
     // additional row info
 
     if rows_remaining > 0 {
-        print!("{: <6}", "");
-        print!(
-            "{}",
-            row_remaining_text.truecolor(meta_color[0], meta_color[1], meta_color[2])
-        );
+        let _ = match stdout!("{: <6}", "") {
+            Ok(_) => Ok(()),
+            Err(e) => match e.kind() {
+                std::io::ErrorKind::BrokenPipe => Ok(()),
+                _ => Err(e),
+            },
+        };
+        if is_tty {
+            let _ = match stdout!(
+                "{}",
+                row_remaining_text.truecolor(meta_color[0], meta_color[1], meta_color[2])
+            ) {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
+        } else {
+            let _ = match stdout!("{}", row_remaining_text) {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
+        }
         //println!("num_cols_to_print {:?} cols {:?}", num_cols_to_print, cols);
         let extra_cols_to_mention = num_cols_to_print;
         let remainder_cols = cols - extra_cols_to_mention;
@@ -456,41 +616,123 @@ fn main() {
             let meta_text_var = "more variables";
             let meta_text_comma = ",";
             let meta_text_colon = ":";
-            print!(
-                " {} {} {}{}",
-                meta_text_and.truecolor(meta_color[0], meta_color[1], meta_color[2]),
-                remainder_cols.truecolor(meta_color[0], meta_color[1], meta_color[2]),
-                meta_text_var.truecolor(meta_color[0], meta_color[1], meta_color[2]),
-                meta_text_colon.truecolor(meta_color[0], meta_color[1], meta_color[2])
-            );
+            if is_tty {
+                let _ = match stdout!(
+                    " {} {} {}{}",
+                    meta_text_and.truecolor(meta_color[0], meta_color[1], meta_color[2]),
+                    remainder_cols.truecolor(meta_color[0], meta_color[1], meta_color[2]),
+                    meta_text_var.truecolor(meta_color[0], meta_color[1], meta_color[2]),
+                    meta_text_colon.truecolor(meta_color[0], meta_color[1], meta_color[2])
+                ) {
+                    Ok(_) => Ok(()),
+                    Err(e) => match e.kind() {
+                        std::io::ErrorKind::BrokenPipe => Ok(()),
+                        _ => Err(e),
+                    },
+                };
+            } else {
+                let _ = match stdout!(
+                    " {} {} {}{}",
+                    meta_text_and,
+                    remainder_cols,
+                    meta_text_var,
+                    meta_text_colon
+                ) {
+                    Ok(_) => Ok(()),
+                    Err(e) => match e.kind() {
+                        std::io::ErrorKind::BrokenPipe => Ok(()),
+                        _ => Err(e),
+                    },
+                };
+            }
             for col in extra_cols_to_mention..cols {
                 let text = rdr[0].get(col).unwrap();
-                print!(
-                    " {}",
-                    text.truecolor(meta_color[0], meta_color[1], meta_color[2])
-                );
+                if is_tty {
+                    let _ = match stdout!(
+                        " {}",
+                        text.truecolor(meta_color[0], meta_color[1], meta_color[2])
+                    ) {
+                        Ok(_) => Ok(()),
+                        Err(e) => match e.kind() {
+                            std::io::ErrorKind::BrokenPipe => Ok(()),
+                            _ => Err(e),
+                        },
+                    };
+                } else {
+                    let _ = match stdout!(" {}", text) {
+                        Ok(_) => Ok(()),
+                        Err(e) => match e.kind() {
+                            std::io::ErrorKind::BrokenPipe => Ok(()),
+                            _ => Err(e),
+                        },
+                    };
+                }
 
                 // The last column mentioned in foot should not be followed by a comma
                 if col + 1 < cols {
-                    print!(
-                        "{}",
-                        meta_text_comma.truecolor(meta_color[0], meta_color[1], meta_color[2])
-                    )
+                    if is_tty {
+                        let _ = match stdout!(
+                            "{}",
+                            meta_text_comma.truecolor(meta_color[0], meta_color[1], meta_color[2])
+                        ) {
+                            Ok(_) => Ok(()),
+                            Err(e) => match e.kind() {
+                                std::io::ErrorKind::BrokenPipe => Ok(()),
+                                _ => Err(e),
+                            },
+                        };
+                    } else {
+                        let _ = match stdout!("{}", meta_text_comma) {
+                            Ok(_) => Ok(()),
+                            Err(e) => match e.kind() {
+                                std::io::ErrorKind::BrokenPipe => Ok(()),
+                                _ => Err(e),
+                            },
+                        };
+                    }
                 }
-            }
+            } // end extra cols mentioned in footer
         }
     }
 
     // footer
     if !datatype::is_na(&footer_option.clone()) {
-        println!("{: <6}", "");
-        println!(
-            "{}",
-            footer_option.truecolor(meta_color[0], meta_color[1], meta_color[2])
-        );
+        let _ = match stdout!("{: <6}", "") {
+            Ok(_) => Ok(()),
+            Err(e) => match e.kind() {
+                std::io::ErrorKind::BrokenPipe => Ok(()),
+                _ => Err(e),
+            },
+        };
+        if is_tty {
+            let _ = match stdoutln!(
+                "{}",
+                footer_option.truecolor(meta_color[0], meta_color[1], meta_color[2])
+            ) {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
+        } else {
+            let _ = match stdoutln!("{}", footer_option) {
+                Ok(_) => Ok(()),
+                Err(e) => match e.kind() {
+                    std::io::ErrorKind::BrokenPipe => Ok(()),
+                    _ => Err(e),
+                },
+            };
+        }
     }
 
-    println!();
+    let _ = match stdoutln!() {
+        Ok(_) => Ok(()),
+        Err(e) => match e.kind() {
+            std::io::ErrorKind::BrokenPipe => Ok(()),
+            _ => Err(e),
+        },
+    };
 } // end main
 
 fn get_color_from_config(a: &toml::value::Array) -> [u8; 3] {
@@ -527,16 +769,31 @@ fn get_num_cols_to_print(cols: usize, vp: Vec<Vec<String>>, term_tuple: (u16, u1
 }
 
 fn build_reader(opt: &Cli) -> Result<Reader<Box<dyn Read>>, std::io::Error> {
-    let source: Box<dyn Read> = if let Some(path) = opt.file.clone() {
+    let mut delimiter = b',';
+
+    let source: Box<dyn Read> = if let Some(path) = &opt.file {
         let file = File::open(path)?;
+
+        // Update the default delimiter by checking the file extension.
+        delimiter = match path.extension() {
+            Some(ext) if ext == "tsv" => b'\t',
+            Some(ext) if ext == "psv" => b'|',
+            _ => delimiter,
+        };
+
         Box::new(BufReader::new(file))
     } else {
         Box::new(io::stdin())
     };
 
+    // Cli options take precedence.
+    if let Some(del) = opt.delimiter {
+        delimiter = del;
+    }
+
     let reader = ReaderBuilder::new()
         .has_headers(false)
-        .delimiter(opt.delimiter)
+        .delimiter(delimiter)
         .from_reader(source);
 
     Ok(reader)
@@ -631,6 +888,7 @@ mod tests {
                 &v[i],
                 col_largest_width_post_proc[i],
                 col_largest_width_post_proc[i],
+                3,
             );
         }
 
@@ -666,7 +924,7 @@ mod tests {
                     "  1           ",
                     " 10           ",
                     "100           ",
-                    "NA            ",
+                    " NA           ",
                     "2/ 2.5 Gallon "
                 ],
                 [
@@ -708,6 +966,7 @@ mod tests {
                 &v[i],
                 col_largest_width_post_proc[i],
                 col_largest_width_post_proc[i],
+                3,
             );
         }
 
@@ -740,6 +999,7 @@ mod tests {
                 &v[i],
                 col_largest_width_post_proc[i],
                 col_largest_width_post_proc[i],
+                3,
             );
         }
 
@@ -763,4 +1023,50 @@ mod tests {
         let reader = build_reader(&cli);
         assert!(reader.is_ok());
     }
+
+    #[test]
+    fn test_is_number() {
+        // Integers
+        assert_eq!(datatype::is_number("12345"), true);
+        assert_eq!(datatype::is_number("   12345"), true);
+        assert_eq!(datatype::is_number("12345   "), true);
+        assert_eq!(datatype::is_number("   12345   "), true);
+        assert_eq!(datatype::is_number("-12345"), true);
+        assert_eq!(datatype::is_number("   -12345"), true);
+        assert_eq!(datatype::is_number("-12345   "), true);
+        assert_eq!(datatype::is_number("   -12345   "), true);
+
+        // Doubles
+        assert_eq!(datatype::is_number("123.45"), true);
+        assert_eq!(datatype::is_number("   123.45"), true);
+        assert_eq!(datatype::is_number("123.45   "), true);
+        assert_eq!(datatype::is_number("   123.45   "), true);
+        assert_eq!(datatype::is_number("0."), true);
+        assert_eq!(datatype::is_number(".0"), true);
+        assert_eq!(datatype::is_number("-123.45"), true);
+        assert_eq!(datatype::is_number("   -123.45"), true);
+        assert_eq!(datatype::is_number("-123.45   "), true);
+        assert_eq!(datatype::is_number("   -123.45   "), true);
+
+        // Misc
+        assert_eq!(datatype::is_number("123text"), false);
+        assert_eq!(datatype::is_number("text123"), false);
+        assert_eq!(datatype::is_number("123.123.123"), false);
+    }
+
+    #[test]
+    fn test_is_negative_number() {
+        assert_eq!(datatype::is_negative_number("-12345"), true);
+        assert_eq!(datatype::is_negative_number("   -12345"), true);
+        assert_eq!(datatype::is_negative_number("-12345   "), true);
+        assert_eq!(datatype::is_negative_number("   -12345   "), true);
+        assert_eq!(datatype::is_negative_number("-12.345"), true);
+        assert_eq!(datatype::is_negative_number("   -12.345"), true);
+        assert_eq!(datatype::is_negative_number("-12.345   "), true);
+        assert_eq!(datatype::is_negative_number("   -12.345   "), true);
+        assert_eq!(datatype::is_negative_number("0.0"), false);
+        assert_eq!(datatype::is_negative_number("0."), false);
+        assert_eq!(datatype::is_negative_number("text"), false);
+        assert_eq!(datatype::is_negative_number("-123.123.123"), false);
+    }
 }
